
/*Program written by Matthew Nelson
   Jan 3, 2016
   for Fresno Ideaworks
   http://fresnoideaworks.org

   If you need help please come to Tech Tuesdays
   or Sumobot Saturdays at Fresno Ideaworks

Suggestions:  
-Experiment with the setSpeed() command to optimize your robot.
-If your motors reverse when told to go forward reverse the +- wires on the motor shield.   
*/

#include <Wire.h>
#include <Adafruit_MotorShield.h>
#include "utility/Adafruit_PWMServoDriver.h"
#include "SRF05.h"
#include <avr/interrupt.h>

// Create the motor shield object with the default I2C address
Adafruit_MotorShield AFMS = Adafruit_MotorShield();

// Select which 'port' M1, M2, M3 or M4 your motors are wired to.
Adafruit_DCMotor *rightmotor = AFMS.getMotor(3); //right motor connected to M3
Adafruit_DCMotor *leftmotor = AFMS.getMotor(4); //left motor connected to M4

// trigPin, echoPin, MaxDist, readInterval
// Trigger is conneccted to pin 5, Echo to 4, 30 cm max distance, read every time through the loop
SRF05 Sensor(5, 4, 30, 0);
const int frontsensorPin = 3;     // front line sensor is connected to pin 3, blue wire
const int rearsensorPin = 2;      // rear line sensor is connected to pin 2, green wire
const int button1Pin = 8;  // The button is hooked up to digital pin 8
const int ledPin =  13;    // LED wire is hooked up to digital pin 13

const int white = 1;
const int black = 0;

int frontsensorState = black;         // when the program starts the front line sensor is seeing black
int rearsensorState = black;          // when the program starts the rear line sensor is seeing black
int button1State = HIGH;  // variable to hold the pushbutton state
boolean turnedon = false;  // Once the inital delay has been completed turned on is set to true
  
void setup()
{
  Serial.begin(9600); // Required to receive debugging info in the Serial Monitor, viewable in the Tools menu
  Serial.println("Fresno Ideaworks Arduinobot starting up...");
  Serial.println("Press the on button to start battle initiation sequence:");
  Sensor.Unlock = true; // Designed to unlock the SRF04 ultrasonic sensor when it locks up
  Sensor.Distance = 0; // Sets the sensor distance to an initial value of zero
  AFMS.begin();  //required for the motor shield

  pinMode(button1Pin, INPUT_PULLUP); // Set up the pushbutton pin to be an input, uses internal pullup resistor
  pinMode(frontsensorPin, INPUT); //Set up the front line sensor pin to be an input
  pinMode(rearsensorPin, INPUT); //Set up the rear line sensor pin to be an input
  pinMode(ledPin, OUTPUT);  // Set up the LED pin to be an output
}

void loop() {

  //Initial startup sequence: On button, LED, and 5 second startup delay
  button1State = digitalRead(button1Pin); // Reads the button state, HIGH is pushed, LOW is not pushed
  if ((button1State == LOW && turnedon == false) ) // if button is pushed AND has not gone through startup delay yet
  {
    Serial.println("Starting 5 second delay...");
    digitalWrite(ledPin, HIGH);  // turn the LED on
    delay(500);                  //leave the LED on for 1/2 of a second to show button was pushed
    digitalWrite(ledPin, LOW);  // turn the LED off
    delay(4500);                  //wait 4.5 seconds with LED off
    digitalWrite(ledPin, HIGH);  // turn the LED on, time to battle!
    turnedon = true;  // Startup delay sequence has been completed!  This loop won't run again.
  }

  while (turnedon == true ) // Do this loop repeatedly after startup delay runs once
  {
    Serial.println("Battle loop entered!"); //Inital startup delay completed, running the battle code


    if ( Sensor.Read() > -1 ) //Get sensor data from front ultrasonic sensor
    {
      // New distance reading!
      if ( Sensor.Distance == 0 ) //Ultrasonic sensor didn't see anything
      {
        Serial.println("No enemy bots detected");
        stopmotors(); //Stops any previously sent commands sent to the motors
        spinright();  //Turn right motor CCW, Turn left motor CW.  Resumes scanning.
      }
      else //This chunk of code runs when the ultrasonic sensor seeings something
      {
        // -------------
        Serial.print("Distance to ememy robot: ");
        Serial.print(Sensor.Distance);
        Serial.println(" cm");
        forward();  //Turn both motors on, bot moves forward
      }
    }

    //If the front line sensor sees white, run the frontwhiteline procedure
    frontsensorState = digitalRead(frontsensorPin);
    if (frontsensorState == white) {
      frontwhiteline();  //stops the motors, reverses the bot
      Serial.println("front white line detected");
    }

    //If the rear line sensor sees white, run the rearwhiteline procedure
    rearsensorState = digitalRead(rearsensorPin);
    if (rearsensorState == white) {
      rearwhiteline(); //stops the bot, bot moves forward
      Serial.println("rear white line detected");
    }

  }//End of the while loop
} //End of the loop function


//This procedure makes the bot move forward
void forward()
{
  Serial.println("Moving forward");
  rightmotor->setSpeed(75); //Sets the right motor speed 0-255
  rightmotor->run(FORWARD);
  leftmotor->setSpeed(75); //Sets the left motor speed 0-255
  leftmotor->run(FORWARD);
}

//This procedure makes the bot move backward
void reverse()
{
  Serial.println("Moving backward");
  rightmotor->setSpeed(75); //Sets the right motor speed 0-255
  rightmotor->run(BACKWARD);
  leftmotor->setSpeed(75); //Sets the left motor speed 0-255
  leftmotor->run(BACKWARD);
}

//This procedure makes the bot move spin to the right when viewed from above
void spinright()
{
  Serial.println("Spinning right");
  rightmotor->setSpeed(55); //Sets the right motor speed 0-255
  rightmotor->run(BACKWARD);
  leftmotor->setSpeed(55); //Sets the left motor speed 0-255
  leftmotor->run(FORWARD);
}

//This procedure stops both motors
void stopmotors()
{
  Serial.println("Stopping both motors");
  rightmotor->run(RELEASE); //turn off the right motor
  leftmotor->run(RELEASE); //turn off the left motor
}

//This procedure runs when the front line sensor sees white
void frontwhiteline()
{
  stopmotors(); //turns off the motors
  delay(1000); //waits one second to stop forward momentum
  reverse(); //back up the bot for 0.8 seconds
  delay(800);
  Serial.println("White line detected in the front");
}

//This procedure runs when the rear line sensor sees white
void rearwhiteline()
{
  stopmotors(); //turns off the motors
  delay(1000); //waits one second to stop rearward momentuum
  forward(); //Make the bot move forward for 0.8 seconds
  delay(800);
  Serial.println("White line detected in the rear");
}


